Imports System
Imports AdventureWorks.Cinema.ServiceAccessLayer
Imports AdventureWorks.Cinema.ServiceAccessLayer.WebService
Imports AdventureWorks.Cinema.SharedBusinessLogic

''' <summary>
''' Der Controller des Model-View-Controller-Entwurfsmusters.
''' </summary>
''' <remarks>
''' Der Controller ist das Hauptprogramm in dieser Beispielanwendung. Er besteht
''' lediglich aus einer Schleife, innerhalb derer er je nach aktuellem Zustand
''' die Sicht (View) synchron aufruft und die Aktionen des Benutzers durch
''' Aufrufe an die Geschftslogik ausfhrt.
''' </remarks>
Friend Class Controller

    Friend Shared Sub Main()

        ' Definiere Variablen fr Verweise auf aktuell ausgewhlte Objekte.
        ' Weise Nothing zu, um Compiler-Warnungen zu unterdrcken.
        ' Erstelle leere Listen, um NullReferenceExceptions zu verhindern.
        Dim selectedNewsItem As NewsItem = Nothing
        Dim selectedMovie As Movie = Nothing
        Dim selectedMovieInfo As MovieInfo = Nothing
        Dim selectedPerformance As Performance = Nothing
        Dim selectedPerformanceInfo As PerformanceInfo = Nothing
        Dim movieInfos As New List(Of MovieInfo)
        Dim tickets As New List(Of Ticket)

        ' Definiere Variablen fr den aktuellen Benutzer und seine Anmeldedaten.
        Dim currentCustomer As User = Nothing
        Dim userName As String = Nothing
        Dim password As String = Nothing

        ' Erstelle eine Instanz der gemeinsamen Geschftslogik.
        Dim cinemaBusiness As New CinemaBusiness

        ' Erstelle die Sicht.
        Dim view As IView = New View()

        ' Lies aus der Geschftslogik den Zustand des Demo-Modus aus und gib ihn
        ' an die Sicht weiter.
        view.IsSampleMode = cinemaBusiness.IsSampleMode

        ' Definiere eine Variable fr den aktuellen Zustand des Controllers.
        Dim state As ControllerStates = ControllerStates.Overview

        ' Definiere Variablen fr den letzen und vorletzten Zustand, um eine
        ' "zurck" Navigation zu ermglichen.
        Dim lastState As ControllerStates = ControllerStates.Overview
        Dim secondLastState As ControllerStates = ControllerStates.Overview

        ' Definiere eine Variable die speichert, ob die Anwendung beendet
        ' werden soll.
        Dim doEndProgram As Boolean = False

        ' Starte die Haupt-Programm-Schleife.
        Do Until doEndProgram

            ' Sichere den letzten und den aktuellen Zustand, um eine "zurck"
            ' Navigation zu ermglichen.
            secondLastState = lastState
            lastState = state

            ' Behandle den aktuellen Zustand.
            Select Case state

                Case ControllerStates.Overview

                    ' Definiere die Ergebnisstruktur.
                    Dim result As OverviewResult

                    ' Rufe die Sicht auf und nimm die Ergebnisstruktur entgegen.
                    result = view.Overview(currentCustomer)

                    ' Werte die Ergebnisstruktur aus.
                    If result.DoEditCustomer Then

                        ' Zeige als nchstes den Zustand EditCustomer an.
                        state = ControllerStates.EditCustomer

                    ElseIf result.DoLogOnCustomer Then

                        ' Zeige als nchstes den Zustand LogOnCustomer an.
                        state = ControllerStates.LogOnCustomer

                    ElseIf result.DoLogOffCustomer Then

                        Try

                            ' Melde den Kunden ab.
                            cinemaBusiness.Customers.LogOff()

                            ' Vergiss alles ber den Kunden.
                            currentCustomer = Nothing
                            userName = Nothing
                            password = Nothing

                            ' Schalte den Demo-Modus ein.
                            cinemaBusiness.IsSampleMode = True
                            view.IsSampleMode = True

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.CustomerNotLoggedOff + _
                                My.Resources.Text.ServiceAccessException)

                        Catch ex As Exception

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.CustomerNotLoggedOff + _
                                My.Resources.Text.UnknownException)

                        End Try

                    ElseIf result.DoShowMovies Then

                        ' Zeige als nchstes den Zustand ShowMovies an.
                        state = ControllerStates.ShowMovies

                        Try

                            ' Rufe die Liste aller Filme ab.
                            movieInfos = cinemaBusiness.Movies.Movies

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MoviesNotLoaded + _
                                My.Resources.Text.ServiceAccessException)

                            ' Erstelle eine leere Liste.
                            movieInfos = New List(Of MovieInfo)

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        Catch ex As Exception

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MoviesNotLoaded + _
                                My.Resources.Text.UnknownException)

                            ' Erstelle eine leere Liste.
                            movieInfos = New List(Of MovieInfo)

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        End Try

                    ElseIf result.DoSearchMoviesByGenres Then

                        ' Zeige als nchstes den Zustand ShowMovies an.
                        state = ControllerStates.ShowMovies

                        Try

                            ' Rufe die Liste aller vom Kunden bevorzugten Filme
                            ' ab.
                            movieInfos = cinemaBusiness.Customers.Movies

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MoviesNotLoaded + _
                                My.Resources.Text.ServiceAccessException)

                            ' Erstelle eine leere Liste.
                            movieInfos = New List(Of MovieInfo)

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        Catch ex As Exception

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MoviesNotLoaded + _
                                My.Resources.Text.UnknownException)

                            ' Erstelle eine leere Liste.
                            movieInfos = New List(Of MovieInfo)

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        End Try

                    ElseIf result.DoShowNews Then

                        ' Zeige als nchstes den Zustand ShowNews an.
                        state = ControllerStates.ShowNews

                    ElseIf result.DoShowPerformances Then

                        ' Vergiss alles ber den aktuellen Film, damit er in
                        ' der Liste der Vorstellungen nicht vorausgewhlt wird.
                        selectedMovie = Nothing
                        selectedMovieInfo = Nothing

                        ' Zeige als nchstes den Zustand ShowPerformances an.
                        state = ControllerStates.ShowPerformances

                    ElseIf result.DoShowTickets Then

                        ' Zeige als nchstes den Zustand ShowTickets an.
                        state = ControllerStates.ShowTickets

                    Else

                        ' Beende die Haupt-Programm-Schleife nach diesem
                        ' Durchlauf.
                        doEndProgram = True

                    End If

                Case ControllerStates.LogOnCustomer

                    ' Erstelle die Ergebnisstruktur und rufe die Sicht auf.
                    Dim result As LogOnCustomerResult
                    result = view.LogOnCustomer( _
                        userName, password, cinemaBusiness.WebServiceAddress)

                    ' Speichere die URL des Webservice.
                    If Not result.WebServiceAddress Is Nothing Then
                        cinemaBusiness.WebServiceAddress = _
                            result.WebServiceAddress
                    End If

                    ' Setze den Demo-Modus.
                    cinemaBusiness.IsSampleMode = result.UseSampleMode
                    view.IsSampleMode = result.UseSampleMode

                    ' Werte die Ergebnisstruktur aus.
                    If Not String.IsNullOrEmpty(result.UserName) Then

                        ' Speichere den Benutzernamen und das Kennwort.
                        userName = result.UserName
                        password = result.Password

                        ' Prfe, ob ein neues Konto erstellt werden soll.
                        If result.DoCreateCustomer Then

                            ' Zeige als nchstes den Zustand EditCustomer an.
                            state = ControllerStates.EditCustomer

                            Try

                                ' Erstelle ein neues Benutzerkonto.
                                cinemaBusiness.Customers.Create( _
                                    userName, password)

                            Catch ex As ArgumentException

                                ' Sag dem Benutzer, dass es ein Problem mit dem
                                ' Benutzernamen oder dem Kennwort gibt.
                                ' Hinweis: die mitgelieferte Sicht berprft die
                                ' Eingabe auch, jedoch knnte eine andere
                                ' Sicht verwendet werden, die diese Prfungen
                                ' nicht durchfhrt. Daher mssen sie hier auf
                                ' jeden Fall durchgefhrt werden.
                                view.Message( _
                                    My.Resources.Text.CustomerNotCreated + _
                                    My.Resources.Text. _
                                    UserNameOrPasswordException)

                            Catch ex As ServiceAccessException

                                ' Sag dem Benutzer, dass ein Webservice
                                ' Zugriffsfehler aufgetreten ist.
                                view.Message( _
                                    My.Resources.Text.CustomerNotCreated + _
                                    My.Resources.Text.ServiceAccessException)

                                ' Zeige als nchstes den Zustand Overview an.
                                state = ControllerStates.Overview

                            Catch ex As Exception

                                ' Sag dem Benutzer, dass ein unbekannter Fehler
                                ' aufgetreten ist.
                                view.Message( _
                                    My.Resources.Text.CustomerNotCreated + _
                                    My.Resources.Text.UnknownException)

                                ' Zeige als nchstes den Zustand Overview an.
                                state = ControllerStates.Overview

                            End Try

                        Else

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        End If

                        Try

                            ' Melde den Kunden an.
                            cinemaBusiness.Customers.LogOn(userName, password)

                            Try

                                ' Rufe die Kundendaten ab.
                                currentCustomer = _
                                    cinemaBusiness.Customers.CurrentCustomer

                            Catch ex As NoUserLoggedOnException

                                ' Sag dem Benutzer, dass er nicht lnger
                                ' angemeldet ist.
                                ' Hinweis: Falls diese Ausnahme hier gefangen
                                ' wird, muss der Fehler in der gemeinsamen
                                ' geschftslogik oder tiefer liegen.
                                view.Message( _
                                    My.Resources.Text.CustomerNotLoaded + _
                                    My.Resources.Text.NoUserLoggedOnException)

                            Catch ex As Exception

                                ' Sag dem Benutzer, dass ein unbekannter Fehler
                                ' aufgetreten ist.
                                view.Message( _
                                    My.Resources.Text.CustomerNotLoaded + _
                                    My.Resources.Text.UnknownException)

                            End Try

                        Catch ex As ArgumentException

                            ' Sag dem Benutzer, dass es ein Problem mit dem
                            ' Benutzernamen oder dem Kennwort gibt.
                            view.Message( _
                                My.Resources.Text.CustomerNotLoggedOn + _
                                My.Resources.Text.UserNameOrPasswordException)

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.LogOnCustomer

                            ' Schalte den Demo-Modus ein.
                            cinemaBusiness.IsSampleMode = True
                            view.IsSampleMode = True

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.CustomerNotLoggedOn + _
                                My.Resources.Text.ServiceAccessException)

                            ' Schalte den Demo-Modus ein.
                            cinemaBusiness.IsSampleMode = True
                            view.IsSampleMode = True

                        Catch ex As Exception

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.CustomerNotLoggedOn + _
                                My.Resources.Text.UnknownException)

                            ' Schalte den Demo-Modus ein.
                            cinemaBusiness.IsSampleMode = True
                            view.IsSampleMode = True

                        End Try

                    Else

                        ' Zeige als nchstes den Zustand Overview an.
                        state = ControllerStates.Overview

                    End If

                Case ControllerStates.EditCustomer

                    Dim genres As List(Of Genre)

                    Try

                        ' Rufe die Liste aller Genres ab.
                        genres = cinemaBusiness.Genres.Genres

                    Catch ex As ServiceAccessException

                        ' Sag dem Benutzer, dass ein Web Service-Zugriffsfehler
                        ' aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.GenresNotLoaded + _
                            My.Resources.Text.ServiceAccessException)

                        ' Erstelle eine leere Liste.
                        genres = New List(Of Genre)

                    Catch ex As Exception

                        ' Sag dem Benutzer, dass ein unbekannter Fehler
                        ' aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.GenresNotLoaded + _
                            My.Resources.Text.UnknownException)

                        ' Erstelle eine leere Liste.
                        genres = New List(Of Genre)

                    End Try

                    ' Rufe die Sicht auf.
                    Dim result As EditCustomerResult = _
                        view.EditCustomer(currentCustomer, genres)

                    ' berprfe, ob der Benutzer das Bearbeiten nicht
                    ' abgebrochen hat.
                    If Not result.Customer Is Nothing Then

                        ' Speichere die alten Kundendaten.
                        Dim backupCustomer As New User
                        With backupCustomer
                            .FirstNames = currentCustomer.FirstNames
                            .LastName = currentCustomer.LastName
                            .Street = currentCustomer.Street
                            .ZipCode = currentCustomer.ZipCode
                            .City = currentCustomer.City
                            .Email = currentCustomer.Email
                            .Newsletter = currentCustomer.Newsletter
                            .Genres = currentCustomer.Genres

                        End With

                        ' Kopiere die neuen Kundendaten in das Kundenobjekt.
                        ' Hinweis: das ist notwendig, damit die nicht
                        ' zugnglichen Felder des Kundenobjektes (wie die Id)
                        ' nicht verloren gehen.
                        With currentCustomer
                            .FirstNames = result.Customer.FirstNames
                            .LastName = result.Customer.LastName
                            .Street = result.Customer.Street
                            .ZipCode = result.Customer.ZipCode
                            .City = result.Customer.City
                            .Email = result.Customer.Email
                            .Newsletter = result.Customer.Newsletter
                            .Genres = result.Customer.Genres
                        End With

                        Try

                            ' Gib die neuen Kundendaten an die Geschftslogik.
                            cinemaBusiness.Customers.Update(currentCustomer)

                        Catch ex As NoUserLoggedOnException

                            ' Sag dem Benutzer, dass er nicht lnger angemeldet
                            ' ist.
                            view.Message( _
                                My.Resources.Text.CustomerNotSaved + _
                                My.Resources.Text.NoUserLoggedOnException)

                            ' Stelle die alten Kundendaten wieder her.
                            With currentCustomer
                                .FirstNames = backupCustomer.FirstNames
                                .LastName = backupCustomer.LastName
                                .Street = backupCustomer.Street
                                .ZipCode = backupCustomer.ZipCode
                                .City = backupCustomer.City
                                .Email = backupCustomer.Email
                                .Newsletter = backupCustomer.Newsletter
                                .Genres = backupCustomer.Genres
                            End With

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.CustomerNotSaved + _
                                My.Resources.Text.UnknownException)

                            ' Stelle die alten Kundendaten wieder her.
                            With currentCustomer
                                .FirstNames = backupCustomer.FirstNames
                                .LastName = backupCustomer.LastName
                                .Street = backupCustomer.Street
                                .ZipCode = backupCustomer.ZipCode
                                .City = backupCustomer.City
                                .Email = backupCustomer.Email
                                .Newsletter = backupCustomer.Newsletter
                            End With

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        Catch ex As Exception

                            ' Die ArgumentException und ArgumentNullException,
                            ' die an dieser Stelle von der Geschftslogik
                            ' geworfen worden sein knnten, werden hier nicht
                            ' explizit gefangen, da sie hier eigentlich nicht
                            ' auftreten knnen. Das Kundenobjekt ist eben erst
                            ' erstellt worden.

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.CustomerNotSaved + _
                                My.Resources.Text.UnknownException)

                            ' Stelle die alten Kundendaten wieder her.
                            With currentCustomer
                                .FirstNames = backupCustomer.FirstNames
                                .LastName = backupCustomer.LastName
                                .Street = backupCustomer.Street
                                .ZipCode = backupCustomer.ZipCode
                                .City = backupCustomer.City
                                .Email = backupCustomer.Email
                                .Newsletter = backupCustomer.Newsletter
                            End With

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        End Try

                    End If

                    ' prfe, ob das Kennword gendert werden soll.
                    If Not String.IsNullOrEmpty(result.Password) Then

                        Try

                            ' ndere das Kennwort.
                            cinemaBusiness.Customers.ChangePassword( _
                                password, result.Password)

                            ' Speichere das neue Kennwort.
                            password = result.Password

                        Catch ex As NoUserLoggedOnException

                            ' Sag dem Benutzer, dass er nicht lnger angemeldet
                            ' ist.
                            view.Message( _
                                My.Resources.Text.PasswordNotChanged + _
                                My.Resources.Text.NoUserLoggedOnException)

                        Catch ex As ArgumentException

                            ' Sag dem Benutzer, dass es ein Problem mit dem
                            ' Benutzernamen oder dem Kennwort gibt.
                            view.Message( _
                                My.Resources.Text.PasswordNotChanged + _
                                My.Resources.Text.UserNameOrPasswordException)

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.PasswordNotChanged + _
                                My.Resources.Text.UnknownException)

                        Catch ex As Exception

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.PasswordNotChanged + _
                                My.Resources.Text.UnknownException)

                        End Try

                    End If

                    ' Zeige als nchstes den Zustand Overview an.
                    state = ControllerStates.Overview

                Case ControllerStates.ShowNews

                    Dim news As List(Of NewsItem)

                    Try

                        ' Rufe die Liste aller Nachrichten ab.
                        news = cinemaBusiness.News.NewsItems

                    Catch ex As ServiceAccessException

                        ' Sag dem Benutzer, dass ein Webservice
                        ' Zugriffsfehler aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.GenresNotLoaded + _
                            My.Resources.Text.ServiceAccessException)

                        ' Erstelle eine leere Liste.
                        news = New List(Of NewsItem)

                    Catch ex As Exception

                        ' Sag dem Benutzer, dass ein unbekannter Fehler
                        ' aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.GenresNotLoaded + _
                            My.Resources.Text.UnknownException)

                        ' Erstelle eine leere Liste.
                        news = New List(Of NewsItem)

                    End Try

                    ' Rufe die Sicht auf.
                    selectedNewsItem = view.ShowNews(news)

                    If Not selectedNewsItem Is Nothing Then

                        ' Zeige als nchstes den Zustand ShowNewsItem an.
                        state = ControllerStates.ShowNewsItem

                    Else

                        ' Zeige als nchstes den Zustand Overview an.
                        state = ControllerStates.Overview

                    End If

                Case ControllerStates.ShowNewsItem

                    ' Rufe die Sicht auf.
                    view.ShowNewsItem(selectedNewsItem)

                    ' Zeige als nchstes den Zustand ShowNews an.
                    state = ControllerStates.ShowNews

                Case ControllerStates.ShowMovies

                    ' Rufe die Sicht auf.
                    selectedMovieInfo = view.ShowMovies(movieInfos)

                    ' Werte die Ergebnisstruktur aus.
                    If Not selectedMovieInfo Is Nothing Then

                        ' Zeige als nchstes den Zustand SearchMovie an.
                        state = ControllerStates.ShowMovie

                        Try

                            ' Rufe die vollstndigen Filmdaten ab.
                            selectedMovie = _
                                cinemaBusiness.Movies.Get(selectedMovieInfo)

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MovieNotLoaded + _
                                My.Resources.Text.ServiceAccessException)

                            ' Vergiss alles ber den Film.
                            selectedMovie = Nothing
                            selectedMovieInfo = Nothing

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        Catch ex As Exception

                            ' Die Geschftslogik kann eine ArgumentNullException
                            ' werden. Das kann hier jedoch nicht passieren.
                            ' Daher wird diese Ausnahme nicht explizit gefangen.

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MovieNotLoaded + _
                                My.Resources.Text.UnknownException)

                            ' Vergiss alles ber den Film.
                            selectedMovie = Nothing
                            selectedMovieInfo = Nothing

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        End Try

                    Else

                        ' Vergiss alles ber den Film.
                        selectedMovie = Nothing

                        ' Zeige als nchstes den Zustand Overview an.
                        state = ControllerStates.Overview

                    End If

                Case ControllerStates.ShowMovie

                    ' Rufe die Sicht auf.
                    Dim doShowPerformances As Boolean = _
                        view.ShowMovie(selectedMovie)

                    ' Werte die Ergebnisstruktur aus.
                    If doShowPerformances Then

                        ' Zeige als nchstes den Zustand ShowPerformances an.
                        state = ControllerStates.ShowPerformances

                    Else

                        ' Gehe in der Navigation zurck.
                        state = secondLastState

                    End If

                Case ControllerStates.ShowPerformances

                    Dim performanceInfos As List(Of PerformanceInfo)

                    Try

                        ' Rufe die Liste aller Vorstellungen ab.
                        performanceInfos = _
                            cinemaBusiness.Performances.Performances

                    Catch ex As ServiceAccessException

                        ' Sag dem Benutzer, dass ein Webservice
                        ' Zugriffsfehler aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.PerformancesNotLoaded + _
                            My.Resources.Text.ServiceAccessException)

                        ' Erstelle eine leere Liste.
                        performanceInfos = New List(Of PerformanceInfo)

                    Catch ex As Exception

                        ' Sag dem Benutzer, dass ein unbekannter Fehler
                        ' aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.PerformancesNotLoaded + _
                            My.Resources.Text.UnknownException)

                        ' Erstelle eine leere Liste.
                        performanceInfos = New List(Of PerformanceInfo)

                    End Try

                    ' Rufe die Sicht auf.
                    Dim result As ShowPerformancesResult = _
                        view.ShowPerformances( _
                        currentCustomer, performanceInfos, selectedMovieInfo)

                    ' Werte die Ergebnisstruktur aus.
                    If Not result.PerformanceInfo Is Nothing Then

                        ' Speichere die ausgewhlte Vorstellung.
                        selectedPerformanceInfo = result.PerformanceInfo

                        ' Zeige als nchstes den Zustand EditPayment an.
                        state = ControllerStates.EditPayment

                        Try

                            ' Rufe die vollstndigen Vorstellungsdaten ab.
                            selectedPerformance = _
                                cinemaBusiness.Performances.Get( _
                                selectedPerformanceInfo)

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.PerformanceNotLoaded + _
                                My.Resources.Text.ServiceAccessException)

                            ' Vergiss alles ber die Vorstellung.
                            selectedPerformance = Nothing
                            selectedPerformanceInfo = Nothing

                            ' Zeige als nchstes den Zustand EditPayment an.
                            state = ControllerStates.EditPayment

                        Catch ex As Exception

                            ' Die ArgumentNullException kann an dieser Stelle
                            ' nicht geworfen worden sein.

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.PerformanceNotLoaded + _
                                My.Resources.Text.UnknownException)

                            ' Vergiss alles ber die Vorstellung.
                            selectedPerformance = Nothing
                            selectedPerformanceInfo = Nothing

                            ' Zeige als nchstes den Zustand EditPayment an.
                            state = ControllerStates.EditPayment

                        End Try

                    ElseIf Not result.MovieInfo Is Nothing Then

                        ' Zeige als nchstes den Zustand ShowMovie an.
                        state = ControllerStates.ShowMovie

                        ' Speichere den ausgewhlten Film.
                        selectedMovieInfo = result.MovieInfo

                        Try

                            ' Rufe die vollstndigen Filmdaten ab.
                            selectedMovie = cinemaBusiness.Movies.Get( _
                                selectedMovieInfo)

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MovieNotLoaded + _
                                My.Resources.Text.ServiceAccessException)

                            ' Vergiss alles ber den Film.
                            selectedMovie = Nothing
                            selectedMovieInfo = Nothing

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        Catch ex As Exception

                            ' Die ArgumentNullException kann an dieser Stelle
                            ' nicht geworfen worden sein.

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.MovieNotLoaded + _
                                My.Resources.Text.UnknownException)

                            ' Vergiss alles ber den Film.
                            selectedMovie = Nothing
                            selectedMovieInfo = Nothing

                            ' Zeige als nchstes den Zustand Overview an.
                            state = ControllerStates.Overview

                        End Try

                    Else

                        ' Zeige als nchstes den Zustand Overview an.
                        state = ControllerStates.Overview

                    End If

                Case ControllerStates.EditPayment

                    ' Rufe die Sicht auf.
                    Dim result As EditPaymentResult = _
                        view.EditPayment(selectedPerformance)

                    ' Werte die Ergebnisstruktur aus.
                    If result.TicketCount > 0 Then

                        ' Zeige als nchstes den Zustand ShowTickets an.
                        state = ControllerStates.ShowTickets

                        Try

                            ' Kaufe das Ticket.
                            cinemaBusiness.Tickets.Buy( _
                                selectedPerformance, result.TicketCount, _
                                result.AccountNumber, result.BankNumber)

                        Catch ex As NoUserLoggedOnException

                            ' Sag dem Benutzer, dass er nicht lnger angemeldet
                            ' ist.
                            view.Message( _
                                My.Resources.Text.TicketsNotBought + _
                                My.Resources.Text.NoUserLoggedOnException)

                            ' Zeige als nchstes den Zustand ShowPerformances
                            ' an.
                            state = ControllerStates.ShowPerformances

                        Catch ex As NotEnoughFreeSeatsException

                            ' Sag dem Benutzer, dass es nicht genug freie
                            ' Pltze gibt.
                            view.Message( _
                                My.Resources.Text.TicketsNotBought + _
                                My.Resources.Text.NotEnoughFreeSeatsException)

                            ' Zeige als nchstes den Zustand ShowPerformances
                            ' an.
                            state = ControllerStates.ShowPerformances

                        Catch ex As ArgumentNullException

                            ' Sag dem Benutzer, dass keine Vorstellung
                            ' ausgewhlt wurde.
                            view.Message( _
                                My.Resources.Text.TicketsNotBought + _
                                My.Resources.Text.NoPerformaceSelectedException)

                            ' Zeige als nchstes den Zustand ShowPerformances
                            ' an.
                            state = ControllerStates.ShowPerformances

                        Catch ex As ArgumentException

                            ' Sag dem Benutzer, dass es ein Problem mit der
                            ' ausgewhlten Anzahl der Tickets gibt.
                            view.Message( _
                                My.Resources.Text.TicketsNotBought + _
                                My.Resources.Text.QuantityException)

                            ' Zeige als nchstes den Zustand ShowPerformances
                            ' an.
                            state = ControllerStates.ShowPerformances

                        Catch ex As ServiceAccessException

                            ' Sag dem Benutzer, dass ein Webservice
                            ' Zugriffsfehler aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.TicketsNotBought + _
                                My.Resources.Text.ServiceAccessException)

                            ' Zeige als nchstes den Zustand ShowPerformances
                            ' an.
                            state = ControllerStates.ShowPerformances

                        Catch ex As Exception

                            ' Sag dem Benutzer, dass ein unbekannter Fehler
                            ' aufgetreten ist.
                            view.Message( _
                                My.Resources.Text.TicketsNotBought + _
                                My.Resources.Text.UnknownException)

                            ' Zeige als nchstes den Zustand ShowPerformances
                            ' an.
                            state = ControllerStates.ShowPerformances

                        End Try

                    Else

                        ' Zeige als nchstes den Zustand ShowPerformances an.
                        state = ControllerStates.ShowPerformances

                    End If

                Case ControllerStates.ShowTickets

                    Try

                        ' Rufe die Liste aller Tickets des Kunden ab.
                        tickets = cinemaBusiness.Customers.Tickets

                    Catch ex As NoUserLoggedOnException

                        ' Sag dem Benutzer, dass er nicht lnger angemeldet ist.
                        view.Message( _
                            My.Resources.Text.TicketsNotLoaded + _
                            My.Resources.Text.NoUserLoggedOnException)

                        ' Zeige als nchstes den Zustand ShowPerformances an.
                        state = ControllerStates.ShowPerformances

                    Catch ex As ServiceAccessException

                        ' Sag dem Benutzer, dass ein Webservice
                        ' Zugriffsfehler aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.TicketsNotLoaded + _
                            My.Resources.Text.ServiceAccessException)

                        ' Zeige als nchstes den Zustand ShowPerformances an.
                        state = ControllerStates.ShowPerformances

                    Catch ex As Exception

                        ' Sag dem Benutzer, dass ein unbekannter Fehler
                        ' aufgetreten ist.
                        view.Message( _
                            My.Resources.Text.TicketsNotLoaded + _
                            My.Resources.Text.UnknownException)

                        ' Zeige als nchstes den Zustand ShowPerformances an.
                        state = ControllerStates.ShowPerformances

                    End Try

                    ' Rufe die Sicht auf.
                    view.ShowTickets(tickets)

                    ' Zeige als nchstes den Zustand Overview an.
                    state = ControllerStates.Overview

            End Select

        Loop

        ' Melde den Kunden falls ntig ab.
        If Not currentCustomer Is Nothing Then
            cinemaBusiness.Customers.LogOff()
        End If

    End Sub

End Class