Imports AdventureWorks.Cinema.ServiceAccessLayer.WebService

''' <summary>
''' Bildschirm fr die Bearbeitung eines Kundenprofils.
''' </summary>
''' <remarks>
''' Dieses Fenster demonstriert wie man Oberflchenelemende zur Laufzeit 
''' erstellt.
''' </remarks>
Friend Class FormEditCustomer

    ''' <summary>
    ''' Property zum Setzen des Status des Demo-Modus.
    ''' </summary>
    Friend WriteOnly Property IsSampleMode() As Boolean
        Set(ByVal value As Boolean)

            ' Setze den Text des Labels passend zum Status des Demo-Modus.
            If value Then
                LabelSampleMode.Text = My.Resources.Text.SampleModeOn
            Else
                LabelSampleMode.Text = My.Resources.Text.SampleModeOff
            End If

        End Set
    End Property

    ''' <summary>
    ''' Ergebnisstruktur.
    ''' </summary>
    Private resultField As EditCustomerResult

    ''' <summary>
    ''' Property zum Auslesen der Ergebnisstruktur.
    ''' </summary>
    Friend ReadOnly Property Result() As EditCustomerResult
        Get
            Return resultField
        End Get
    End Property

    ' Erstelle eine Liste fr die zur Laufzeit erstellten CheckBoxen.
    Private checkBoxes As New List(Of CheckBox)

    ''' <summary>
    ''' Property zum Setzen der Genreliste.
    ''' </summary>
    ''' <remarks>
    ''' Diese Property mu vor der Customer Property gesetzt werden!
    ''' </remarks>
    Friend WriteOnly Property Genres() As List(Of Genre)
        Set(ByVal value As List(Of Genre))

            ' Definiere Variable fr eine neue CheckBox.
            Dim checkBox As CheckBox

            ' Definiere Variable fr die aktuelle vertikale Position.
            Dim y As Integer = 0

            ' Definiere Variable fr die Hhe und Breite einer CheckBox und lies
            ' die Werte aus einem vom Visual Studio generierten Labels aus.
            Dim yOffset As Integer = LabelName.Height
            Dim width As Integer = LabelName.Width

            ' Bearbeite alle Genres.
            For Each genre As Genre In value

                ' Erstelle eine neue CheckBox und setze den Text.
                checkBox = New CheckBox
                checkBox.Text = genre.Name

                ' Setze die Position und Breite der Checkbox.
                checkBox.Location = New Point(0, y)
                checkBox.Width = width

                ' Setze die Ankerpunkte, so da die CheckBox ihre Gre anpasst,
                ' wenn eine Bildlaufleiste ein- oder ausgebledet wird oder die
                ' orientierung des Fensters gendert wird.
                checkBox.Anchor = AnchorStyles.Top Or AnchorStyles.Left Or _
                    AnchorStyles.Right

                ' Verweise auf das Genre.
                checkBox.Tag = genre

                ' Fge die CheckBox den Panel hinzu.
                TabPageGenres.Controls.Add(checkBox)

                ' Fge die CheckBox der Liste hinzu.
                checkBoxes.Add(checkBox)

                ' Erhhe die vertikale Position um die gespeicherte Hhe.
                y = y + yOffset

            Next
        End Set
    End Property

    ''' <summary>
    ''' Property zum Setzen des Kundens.
    ''' </summary>
    ''' <remarks>
    ''' Diese Property muss nach der Genres Property gesetzt werden!
    ''' </remarks>
    Friend WriteOnly Property Customer() As User
        Set(ByVal value As User)

            ' Setze die Texte der Eingabefelder fr die Kundendaten.
            With value

                TextBoxFirstName.Text = .FirstNames
                TextBoxLastName.Text = .LastName
                TextBoxStreet.Text = .Street
                TextBoxZipCode.Text = .ZipCode
                TextBoxCity.Text = .City
                TextBoxEmail.Text = .Email
                CheckBoxNewsletter.Checked = .Newsletter

                ' Kreuze alle CheckBoxen an, die auf ein Genre aus der Liste
                ' der bevorzugten Genres des Kunden verweisen.
                If Not .Genres Is Nothing Then
                    For Each checkBox As CheckBox In checkBoxes
                        For Each genre As Genre In .Genres
                            checkBox.Checked = _
                                    (genre.Id = CType(checkBox.Tag, Genre).Id)
                            If checkBox.Checked Then
                                Exit For
                            End If
                        Next
                    Next
                End If

            End With

        End Set
    End Property

    ''' <summary>
    ''' Ereignisbehandlung fr die drei Eingabefelder fr das alte und neue
    ''' Kennwort.
    ''' </summary>
    Private Sub TextBoxPassword_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxNewPassword.TextChanged, TextBoxNewPasswordConfirmation.TextChanged, TextBoxOldPassword.TextChanged

        ' Deaktiviere den Button zum Speichern, falls mindestens eines der drei
        ' Felder nicht ausgefllt wurden oder falls die Felder fr das neue
        ' Passwort nicht den selben Text enthalten.
        ButtonSave.Enabled = _
            ((String.IsNullOrEmpty(TextBoxOldPassword.Text) And _
            String.IsNullOrEmpty(TextBoxNewPassword.Text) And _
            String.IsNullOrEmpty(TextBoxNewPasswordConfirmation.Text)) Or _
            ((Not String.IsNullOrEmpty(TextBoxOldPassword.Text) And _
            Not String.IsNullOrEmpty(TextBoxNewPassword.Text) And _
            TextBoxNewPassword.Text = TextBoxNewPasswordConfirmation.Text)))

    End Sub

    ''' <summary>
    ''' Ereignisbehandlung fr den Button zum Speichern.
    ''' </summary>
    Private Sub ButtonSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSave.Click

        ' Erstelle einen neuen leeren Kunden in der Ergebnisstruktur.
        resultField.Customer = New User

        ' Speichere die Benutzereingaben in der Ergebnisstruktur.
        With resultField.Customer
            .FirstNames = TextBoxFirstName.Text
            .LastName = TextBoxLastName.Text
            .Street = TextBoxStreet.Text
            .ZipCode = TextBoxZipCode.Text
            .City = TextBoxCity.Text
            .Email = TextBoxEmail.Text
            .Newsletter = CheckBoxNewsletter.Checked
        End With

        ' Zhle die Anzahl der ausgewhlten Genres.
        Dim count As Integer = 0
        For Each checkBox As CheckBox In checkBoxes
            If checkBox.Checked Then
                count = count + 1
            End If
        Next

        ' Erstelle das neue Genre Array in der Ergebnisstruktur.
        resultField.Customer.Genres = New Genre(count - 1) {}

        ' Fge alle ausgewhlten Genres in das Array ein.
        Dim i As Integer = 0
        For Each checkBox As CheckBox In checkBoxes
            If checkBox.Checked Then

                ' Lies das Objekt aus, auf das die aktuelle CheckBox verweist,
                ' wandle es in ein Genre um und fge es in das Array ein.
                resultField.Customer.Genres(i) = CType(checkBox.Tag, Genre)

                i += 1

            End If
        Next

        ' Speichere das neue Kennwort in der Ergebnisstruktur.
        resultField.Password = TextBoxNewPassword.Text

        ' Schliee dieses Fenster.
        Me.Close()

    End Sub

End Class