''' <summary>
''' Bildschirm fr die Eingabe der Anmeldedaten.
''' </summary>
Friend Class FormLogOnCustomer

    ''' <summary>
    ''' Property zum Setzen des Status des Demo-Modus.
    ''' </summary>
    Friend WriteOnly Property IsSampleMode() As Boolean
        Set(ByVal value As Boolean)

            ' Setze den Text des Labels und den RadioButton passend zum Status
            ' des Demo-Modus.
            RadioButtonSampleMode.Checked = value
            TextBoxWebServiceAddress.Enabled = Not value

        End Set
    End Property

    ''' <summary>
    ''' Feld fr den Benutzernamen.
    ''' </summary>
    Private userNameField As String

    ''' <summary>
    ''' Feld fr das Kennwort.
    ''' </summary>
    Private passwordField As String

    ''' <summary>
    ''' Feld fr die URL des Webservice.
    ''' </summary>
    Private webServiceAddressBackup As Uri

    ''' <summary>
    ''' Property zum Setzen und Auslesen des Benutzernamens.
    ''' </summary>
    Friend Property UserName() As String
        Get
            Return userNameField
        End Get
        Set(ByVal value As String)

            ' Setze den Text des Eingabefeldes fr den Benutzernamen.
            TextBoxUserName.Text = value

        End Set
    End Property

    ''' <summary>
    ''' Property zum Setzen und Auslesen des Kennwortes.
    ''' </summary>
    Friend Property Password() As String
        Get
            Return passwordField
        End Get
        Set(ByVal value As String)

            ' Setze den Text des Eingabefeldes fr das Kennwort.
            TextBoxPassword.Text = value

        End Set
    End Property

    ''' <summary>
    ''' Property zum Setzen und Auslesen der URL des Webservice.
    ''' </summary>
    Friend Property WebServiceAddress() As Uri
        Get

            ' Prfe, ob die Benutzereingabe eine gltige Adresse darstellt.
            If (Uri.IsWellFormedUriString(TextBoxWebServiceAddress.Text, _
                UriKind.Absolute)) Then

                ' Erstelle eine Adresse aus der Benutzereingabe.
                Dim webServiceAddressResult As Uri = _
                    New Uri(TextBoxWebServiceAddress.Text)

                ' Gib die Adresse zurck.
                Return webServiceAddressResult

            Else

                ' Gib die gespeicherte Adresse zurck.
                Return webServiceAddressBackup

            End If


        End Get
        Set(ByVal value As Uri)

            ' Speichere die alte Adresse.
            webServiceAddressBackup = value

            ' Setze den Text des Eingabefeldes fr die Adresse des Webservice.
            If Not value Is Nothing Then
                TextBoxWebServiceAddress.Text = value.ToString
            End If

        End Set
    End Property

    ''' <summary>
    ''' Ergebnisstruktur.
    ''' </summary>
    Private resultField As LogOnCustomerResult

    ''' <summary>
    ''' Property fr das Auslesen der Ergebnisstruktur.
    ''' </summary>
    Friend ReadOnly Property Result() As LogOnCustomerResult
        Get
            Return resultField
        End Get
    End Property

    ''' <summary>
    ''' Ereignisbehandlung fr den Button zum Anmelden des Kundens.
    ''' </summary>
    Private Sub ButtonLogOnCustomer_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles ButtonLogOnCustomer.Click

        ' Lies den Benutzernamen und das Kennwort aus den Eingabefeldern aus und
        ' speichere sie in der Ergebnisstruktur.
        resultField.UserName = TextBoxUserName.Text
        resultField.Password = TextBoxPassword.Text

        ' Speichere die Adresse des Webservice.
        resultField.WebServiceAddress = WebServiceAddress

        ' Speichere die Auswahl fr den Demo-Modus.
        resultField.UseSampleMode = RadioButtonSampleMode.Checked

        ' Schliee dieses Fenster.
        Me.Close()

    End Sub

    ''' <summary>
    ''' Ereignisbehandlung fr den Button zum Erstellen eines Kundenkontos.
    ''' </summary>
    Private Sub ButtonCreateCustomer_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles ButtonCreateCustomer.Click

        ' Lies den Benutzernamen und das Kennwort aus den Eingabefeldern aus und
        ' speichere sie in der Ergebnisstruktur.
        resultField.UserName = TextBoxUserName.Text
        resultField.Password = TextBoxPassword.Text

        ' Speichere die Adresse des Webservice.
        resultField.WebServiceAddress = WebServiceAddress

        ' Speichere die Auswahl fr den Demo-Modus.
        resultField.UseSampleMode = RadioButtonSampleMode.Checked

        ' Setze die Markierung.
        resultField.DoCreateCustomer = True

        ' Schliee dieses Fenster.
        Me.Close()

    End Sub

    ''' <summary>
    ''' Ereignisbehandlung fpr die Hilfe zum Demo-Modus.
    ''' </summary>
    Private Sub LinkLabelHelpSampleMode_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles LinkLabelSampleMode.Click

        ' Zeige den Hilfetext an.
        MsgBox(My.Resources.Text.HelpSampleMode, MsgBoxStyle.MsgBoxHelp)

    End Sub

    ''' <summary>
    ''' Ereignisbehandlung fr die Auswahl des Demo-Modus.
    ''' </summary>
    Private Sub RadioButton2_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadioButtonWebService.CheckedChanged

        ' Aktiviere das Eingabefeld fr die Adresse des Webservice nur dann,
        ' wenn der Demo-Modus nicht ausgewhlt wurde.
        TextBoxWebServiceAddress.Enabled = RadioButtonWebService.Checked

    End Sub

End Class