Imports System.Text
Imports System.Globalization
Imports Microsoft.VisualBasic
Imports AdventureWorks.Cinema.ServiceAccessLayer.WebService

''' <summary>
''' Bildschirm fr die Anzeige eines Filmes.
''' </summary>
Friend Class FormShowMovie

    ''' <summary>
    ''' Property zum Setzen des Status des Demo-Modus.
    ''' </summary>
    Friend WriteOnly Property IsSampleMode() As Boolean
        Set(ByVal value As Boolean)

            ' Setze den Text des Labels passend zum Status des Demo-Modus.
            If value Then
                LabelSampleMode.Text = My.Resources.Text.SampleModeOn
            Else
                LabelSampleMode.Text = My.Resources.Text.SampleModeOff
            End If

        End Set
    End Property

    ''' <summary>
    ''' Ergebnisfeld.
    ''' </summary>
    Private doShowPerformancesField As Boolean

    ''' <summary>
    ''' Property zum Auslesen der Ergebnisfeldes.
    ''' </summary>
    Friend ReadOnly Property DoShowPerformances() As Boolean
        Get
            Return doShowPerformancesField
        End Get
    End Property

    ''' <summary>
    ''' Property zum Setzen des Filmes.
    ''' </summary>
    Friend WriteOnly Property Movie() As Movie
        Set(ByVal value As Movie)
            With (value)

                ' Setze den Text des Labels fr den Filmtitel, falls der
                ' Filmtitel in dem Filmobjekt vorhanden ist.
                If Not String.IsNullOrEmpty(.Title) Then
                    LabelTitle.Text = .Title
                End If

                ' Erstelle einen StringBuilder fr die Filmdetails.
                Dim builder As New StringBuilder

                ' Fge den Originaltitel hinzu, falls vorhanden.
                If Not String.IsNullOrEmpty(.OriginalTitle) Then
                    ' Fge Beschreibung hinzu.
                    builder.Append(My.Resources.Text.OriginalTitle + ": ")
                    ' Fge den Originaltitel hinzu.
                    builder.Append(.OriginalTitle)
                    ' Fge einen Zeilenumbruch hinzu.
                    builder.Append(vbNewLine)
                End If

                ' Fge Land und Jahr hinzu, falls vorhanden.
                If Not .Country Is Nothing Or .Year <> 0 Then
                    If Not String.IsNullOrEmpty(.Country.Name) Then
                        builder.Append(.Country.Name)
                    End If
                    ' Fge nur dann ein Komma hinzu, wenn sowohl Land als auch
                    ' jahr vorhanden sind.
                    If Not String.IsNullOrEmpty(.Country.Name) And _
                        .Year <> 0 Then
                        builder.Append(", ")
                    End If
                    If Not .Year <> 0 Then
                        builder.Append(.Year.ToString)
                    End If
                End If

                ' Fge die Genres hinzu, falls vorhanden.
                If Not .Genres Is Nothing AndAlso .Genres.Length > 0 Then
                    builder.Append(" (")
                    For i As Integer = 0 To .Genres.Length - 1
                        builder.Append(.Genres(i).Name)
                        ' Fge ein Komma nach allen Genres auer dem letzen
                        ' hinzu.
                        If i < .Genres.Length - 1 Then
                            builder.Append(", ")
                        End If
                    Next
                    builder.Append(")")
                End If

                ' Fge einen Zeilenumbruch hinzu, falls bisher schon irgendein
                ' Text hinzugefgt wurde.
                If builder.Length > 0 Then
                    builder.Append(vbNewLine)
                End If

                ' Fge die Filmbeschreibung hinzu, falls vorhanden.
                If Not String.IsNullOrEmpty(.Description) Then
                    builder.Append(vbNewLine)
                    builder.Append(.Description)
                End If

                ' Fge die Schauspieler hinzu, falls vorhanden.
                If Not .Actors Is Nothing AndAlso .Actors.Length > 0 Then
                    builder.Append(vbNewLine)
                    builder.Append(My.Resources.Text.WithActors + " ")
                    For i As Integer = 0 To .Actors.Length - 1
                        builder.Append(.Actors(i).Name)
                        ' Fge ein Komma nach allen Schauspielern auer den
                        ' letzten beiden hinzu. Fge ein "und" zwischen dem 
                        ' vorletzten und letzten Schauspieler hinzu.
                        If i < .Actors.Length - 2 Then
                            builder.Append(", ")
                        ElseIf i < .Actors.Length - 1 Then
                            builder.Append(" ")
                            builder.Append(My.Resources.Text.AndActor)
                            builder.Append(" ")
                        End If
                    Next
                    builder.Append(vbNewLine)
                End If

                ' Fge die Filmlnge hinzu, falls vorhanden.
                If .Length <> 0 Then
                    builder.Append(vbNewLine)
                    builder.Append(My.Resources.Text.Length + ": ")
                    builder.Append(.Length.ToString(NumberFormatInfo.CurrentInfo))
                    builder.Append(" " + My.Resources.Text.Minutes)
                End If

                ' Fge die Altersfreigabe hinzu, falls vorhanden.
                If Not .ParentalGuidance Is Nothing Then
                    builder.Append(vbNewLine)
                    builder.Append(My.Resources.Text.ParentalGuidance + ": ")
                    builder.Append(.ParentalGuidance.Name)
                End If

                ' Setze den Text des Labels fr die Filmbeschreibung.
                TextMovie.Text = builder.ToString

            End With
        End Set
    End Property

    ''' <summary>
    ''' Ereignisbehandlung fr den Button zum Anzeigen der Vorstellungen.
    ''' </summary>
    Private Sub ButtonShowPerformances_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles ButtonShowPerformances.Click

        ' Setze das Ergebnisfeld.
        doShowPerformancesField = True

        ' Schliee dieses Fenster.
        Me.Close()

    End Sub

End Class