Imports System.Globalization
Imports AdventureWorks.Cinema.ServiceAccessLayer.WebService

''' <summary>
''' Bildschirm fr die Anzeige einer Film- und Vorstellungsliste.
''' </summary>
Friend Class FormShowPerformances

    ''' <summary>
    ''' Property zum Setzen des Status des Demo-Modus.
    ''' </summary>
    Friend WriteOnly Property IsSampleMode() As Boolean
        Set(ByVal value As Boolean)

            ' Setze den Text des Labels passend zum Status des Demo-Modus.
            If value Then
                LabelSampleMode.Text = My.Resources.Text.SampleModeOn
            Else
                LabelSampleMode.Text = My.Resources.Text.SampleModeOff
            End If

        End Set
    End Property

    ''' <summary>
    ''' Property zum Setzen des aktuellen Kundens.
    ''' </summary>
    Friend WriteOnly Property Customer() As User
        Set(ByVal value As User)

            ButtonBuyTickets.Visible = Not value Is Nothing

        End Set
    End Property

    ''' <summary>
    ''' Speicher fr die Filme zu den Vorstellungen (Filmliste).
    ''' </summary>
    Private movieInfos As New List(Of MovieInfo)

    ''' <summary>
    ''' Property zum Setzen der Vorstellungsliste.
    ''' </summary>
    ''' <remarks>
    ''' Diese Property mu vor der MovieInfo Property gesetzt werden.
    ''' </remarks>
    Friend WriteOnly Property PerformanceInfos() As List(Of PerformanceInfo)
        Set(ByVal value As List(Of PerformanceInfo))

            ' Definiere Index Variable fr den Zugriff auf einzelne Filme in der
            ' Filmliste.
            Dim index As Integer

            ' Definiere Variablen fr das Hinzufgen neuer Knoten zum Baum
            ' (Filmknoten und Vorstellungsknoten).
            Dim performanceNode As TreeNode
            Dim movieNode As TreeNode

            ' Bearbeite alle bergebenen Vorstellungen.
            If Not value Is Nothing Then
                For Each performanceInfo As PerformanceInfo In value

                    ' Finde den Index des Filmes der aktuellen Vorstellung in
                    ' der Filmliste. Da die Webservice Zugriffsschicht keine
                    ' Objekt-Identitt bereitstellt, mssen die Ids verglichen
                    ' werden.
                    index = -1
                    For Each movieInfo As MovieInfo In movieInfos
                        If movieInfo.Id = performanceInfo.MovieInfo.Id Then
                            index = movieInfos.IndexOf(movieInfo)
                            Exit For
                        End If
                    Next

                    ' Prfe ob der Film nicht in der Liste gefunden wurde.
                    If index = -1 Then

                        ' Setze den Index auf das Ende der Liste.
                        index = movieInfos.Count

                        ' Fge den Film ein.
                        movieInfos.Add(performanceInfo.MovieInfo)

                        ' Erstelle einen neuen Filmknoten und verweise auf den
                        ' Film.
                        movieNode = New TreeNode
                        movieNode.Text = performanceInfo.MovieInfo.Title
                        movieNode.Tag = performanceInfo.MovieInfo

                        ' Fge den Filmknoten der Wurzel des Baumes hinzu.
                        TreeViewPerformances.Nodes.Add(movieNode)

                    Else

                        ' Lies den vorhandenen Filmknoten aus dem Baum aus.
                        movieNode = TreeViewPerformances.Nodes(index)

                    End If

                    ' Erstelle einen neuen Vorstellungsknoten und verweise auf
                    ' die Vorstellung.
                    performanceNode = New TreeNode
                    performanceNode.Text = performanceInfo.Date.ToString( _
                        DateTimeFormatInfo.CurrentInfo) + " " + _
                        performanceInfo.RoomName
                    performanceNode.Tag = performanceInfo

                    ' Fge den Vorstellungsknoten dem Filmknoten hinzu.
                    movieNode.Nodes.Add(performanceNode)

                Next
            End If

        End Set
    End Property

    ''' <summary>
    ''' Property zum Setzen des ausgewhlten Filmes.
    ''' </summary>
    ''' <remarks>
    ''' Diese Property mu nach der PerformanceInfos Property gesetzt werden.
    ''' </remarks>
    Friend WriteOnly Property MovieInfo() As MovieInfo
        Set(ByVal value As MovieInfo)

            If Not value Is Nothing Then

                ' Suche den Index des Films in der Filmliste. Dieser entspricht
                ' dem Index des Filmknotens in dem Baum.
                Dim index As Integer = -1
                For Each movieInfo As MovieInfo In movieInfos
                    If movieInfo.Id = value.Id Then
                        index = movieInfos.IndexOf(movieInfo)
                        Exit For
                    End If
                Next

                ' Wenn der Film in dem Baum vorhanden ist, dann klappe den
                ' Knoten aus.
                If index <> -1 Then
                    TreeViewPerformances.Nodes(index).Expand()
                End If

            End If

        End Set
    End Property

    ''' <summary>
    ''' Ergebnisstruktur.
    ''' </summary>
    Private resultField As ShowPerformancesResult

    ''' <summary>
    ''' Property zum Auslesen der Ergebnisstruktur.
    ''' </summary>
    Friend ReadOnly Property Result() As ShowPerformancesResult
        Get
            Return resultField
        End Get
    End Property

    ''' <summary>
    ''' Ereignisbehandlung fr den Button zum Kaufen der Tickets.
    ''' </summary>
    Private Sub ButtonBuyTickets_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles ButtonBuyTickets.Click

        ' Lies den ausgewhlten Baumknoten und versuche ihn in eine Vorstellung
        ' umzuwandeln. Speichere die Vorstellung in der Ergebnisstruktur.
        resultField.PerformanceInfo = _
            TryCast(TreeViewPerformances.SelectedNode.Tag, PerformanceInfo)

        ' Schliee dieses Fenster.
        Me.Close()

    End Sub

    ''' <summary>
    ''' Ereignisbehandlung fr den Button zum Anzeigen eines Filmes.
    ''' </summary>
    Private Sub ButtonShowMovie_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles ButtonShowMovie.Click

        ' Lies den ausgewhlten Baumknoten und versuche ihn in einen Film
        ' umzuwandeln. Speichere den Film in der Ergebnisstruktur.
        resultField.MovieInfo = TryCast( _
            TreeViewPerformances.SelectedNode.Tag, MovieInfo)

        ' Schliee dieses Fenster.
        Me.Close()

    End Sub

    ''' <summary>
    ''' Ereignisbehandlung fr die Auswahl eines Baumknotens.
    ''' </summary>
    Private Sub TreeViewPerformances_AfterSelect(ByVal sender As _
        System.Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) _
        Handles TreeViewPerformances.AfterSelect

        ' Aktiviere den Button zum Kaufen der Tickets nur dann, wenn eine
        ' Vorstellung ausgewhlt wurde.
        ButtonBuyTickets.Enabled = _
            TypeOf TreeViewPerformances.SelectedNode.Tag Is PerformanceInfo

        ' Aktiviere den Button zum Anzeigen eines Filmes nur dann, wenn ein
        ' Film ausgewhlt wurde.
        ButtonShowMovie.Enabled = _
            TypeOf TreeViewPerformances.SelectedNode.Tag Is MovieInfo

    End Sub

End Class