Imports AdventureWorks.Cinema.ServiceAccessLayer.WebService

''' <summary>
''' Schnittstelle fr die Sichten des Model-View-Controller-Entwurfsmusters.
''' </summary>
''' <remarks>
''' Diese Schnittstelle enthlt Methoden, um die in dieser Beispielanwendung vom
''' Controller verwendete Sicht (View) fr verschiedene Zwecke aufzurufen.
''' Eingabeparameter werden als Werte bergeben (by value). Ausgaben (wie die
''' Auswahl des Benutzers aus einer Liste) werden als Ergebnis von der Methode
''' zurckgegeben. Diese werden wenn ntig in einer Ergebnisstruktur
''' zusammengefasst (siehe Modell). Parameter, die von der Sicht verndert
''' werden sollen, sind gleichzeitig Ein- und Ausgabeparameter. Sie werden nicht
''' als Referenz bergeben, damit der Controller die nderungen berprfen und
''' verwerfen kann. Alle Methoden entsprechen direkt einem Controller-Zustand.
''' </remarks>
Friend Interface IView

    ''' <summary>
    ''' Legt innerhalb der Sicht fest, ob der Demo-Modus benutzt werden soll.
    ''' </summary>
    ''' <value>Soll der Demo-Modus benutzt werden?</value>
    WriteOnly Property IsSampleMode() As Boolean

    ''' <summary>
    ''' Zeigt eine Nachricht an.
    ''' </summary>
    ''' <param name="text">Nachrichtentext</param>
    Sub Message(ByVal text As String)

    ''' <summary>
    ''' Zeigt den Bildschirm fr das Bearbeiten der Kundendaten an.
    ''' </summary>
    ''' <param name="customer">Aktueller Kunde</param>
    ''' <param name="genres">Liste aller Genres</param>
    ''' <returns>Genderte Kundendaten oder Nothing zum Abbrechen</returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer nur zum Zustand Overview zurckgehen.
    ''' </remarks>
    Function EditCustomer(ByVal customer As User, _
        ByVal genres As List(Of Genre)) As EditCustomerResult

    ''' <summary>
    ''' Zeigt den Bildschirm fr die Eingabe der Bezahlungsmglichkeiten.
    ''' </summary>
    ''' <param name="performance">Vorstellung fr die ein Ticket gekauft
    ''' werden soll</param>
    ''' <returns>Ergebnisstruktur</returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden ShowTickets und Overview
    ''' gehen.
    ''' </remarks>
    Function EditPayment(ByVal performance As Performance) As EditPaymentResult

    ''' <summary>
    ''' Zeigt den Bildschirm fr die Eingabe der Anmeldedaten an.
    ''' </summary>
    ''' <param name="userName">Benutername des aktuellen Kunden</param>
    ''' <param name="password">Kennwort des aktuellen Kunden</param>
    ''' <returns>Ergebnisstruktur</returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden EditCustomer und
    ''' Overview gehen.
    ''' </remarks>
    Function LogOnCustomer(ByVal userName As String, ByVal password As String, _
        ByVal webServiceAddress As Uri) As LogOnCustomerResult

    ''' <summary>
    ''' Zeigt den bersichtsbildschirm an.
    ''' </summary>
    ''' <param name="customer">Aktueller Kunde</param>
    ''' <returns>Ergebnisstruktur</returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden EditCustomer
    ''' LogOnCustomer, ShowMovies, ShowNews, ShowPerformances und ShowTickets
    ''' gehen und die Anwendung beenden.
    ''' </remarks>
    Function Overview(ByVal customer As User) As OverviewResult

    ''' <summary>
    ''' Zeigt den Bildschirm fr Filmdetails an.
    ''' </summary>
    ''' <param name="movie">Film der angezeigt werden soll</param>
    ''' <returns>
    ''' Sollen alle Vorstellungen des Filmes angezeigt werden?
    ''' </returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden ShowMovies und
    ''' ShowPerformances gehen.
    ''' </remarks>
    Function ShowMovie(ByVal movie As Movie) As Boolean

    ''' <summary>
    ''' Zeigt den Bildschirm fr eine Filmliste.
    ''' </summary>
    ''' <param name="movies">
    ''' Liste der Filme die angezeigt werden sollen
    ''' </param>
    ''' <returns>Ausgewhlter Film oder Nothing zum Abbrechen</returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden ShowMovies und Overview
    ''' gehen.
    ''' </remarks>
    Function ShowMovies(ByVal movies As List(Of MovieInfo)) As MovieInfo

    ''' <summary>
    ''' Zeigt den Bildschirm fr eine Nachrichtenliste an.
    ''' </summary>
    ''' <param name="NewsItem">
    ''' Liste der Nachrichten die angezeigt werden soll
    ''' </param>
    ''' <returns>Ausgewhlter Nachricht oder Nothing zum Abbrechen</returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden ShowNewsItem und
    ''' Overview gehen.
    ''' </remarks>
    Function ShowNews(ByVal NewsItem As List(Of NewsItem)) As NewsItem

    ''' <summary>
    ''' Zeigt den Bildschirm fr eine Nachricht an.
    ''' </summary>
    ''' <param name="newsItem">Nachricht die angezeigt werden soll</param>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer nur zum Zustand ShowNews zurckgehen.
    ''' </remarks>
    Sub ShowNewsItem(ByVal newsItem As NewsItem)

    ''' <summary>
    ''' Zeigt den Bildschirm fr eine Vorstellungsliste an.
    ''' </summary>
    ''' <param name="customer">Aktueller Kunde</param>
    ''' <param name="performances">
    ''' Liste der Vorstellungen die angezeigt werden soll
    ''' </param>
    ''' <param name="movieInfo">Film der vorausgewhlt werden soll</param>
    ''' <returns>Ergebnisstruktur</returns>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden ShowMovies und
    ''' EditPayment gehen.
    ''' </remarks>
    Function ShowPerformances(ByVal customer As User, _
        ByVal performances As List(Of PerformanceInfo), _
        ByVal movieInfo As MovieInfo) As ShowPerformancesResult

    ''' <summary>
    ''' Zeigt den Bildschirm fr eine Ticketliste an.
    ''' </summary>
    ''' <param name="tickets">
    ''' Liste der Tickets die angezeigt werden soll
    ''' </param>
    ''' <remarks>
    ''' Von hier aus kann der Benutzer zu den Zustnden ShowMovie und Overview
    ''' gehen.
    ''' </remarks>
    Sub ShowTickets(ByVal tickets As List(Of Ticket))

End Interface