Imports AdventureWorks.Cinema.ServiceAccessLayer.WebService
Imports AdventureWorks.Cinema.SharedBusinessLogic

''' <summary>
''' Aufzhlung aller mglichen Controller-Zustnde.
''' </summary>
''' <remarks>
''' Ein Zustand entspricht direkt einer Methode um die Sicht aufzurufen (siehe
''' IView Schnittstelle).
''' </remarks>
Friend Enum ControllerStates

    ' Zustand fr die Eingabe der Bezahlungsmglichkeiten beim Ticket-Kauf.
    EditPayment

    ' Zustand fr das Bearbeiten der Kundendaten.
    EditCustomer

    ' Zustand fr die Eingabe der Anmeldedaten.
    LogOnCustomer

    ' Zustand fr den Hauptbildschirm der Anwendung.
    Overview

    ' Zustand fr die Anzeige einer Liste von Filmen.
    ShowMovies

    ' Zustand fr die Anzeige der Details eines Filmes.
    ShowMovie

    ' Zustand fr die Anzeige einer Liste von Nachrichten.
    ShowNews

    ' Zustand fr die Anzeige einer Nachricht.
    ShowNewsItem

    ' Zustand fr die Anzeige einer Liste von Filmen und Vorstellungen.
    ShowPerformances

    ' Zustand fr die Anzeige einer Liste von Tickets.
    ShowTickets

End Enum

''' <summary>
''' Ergebnisstruktur fr den Zustand EditCustomer.
''' </summary>
Friend Structure EditCustomerResult
    Friend Customer As User
    Friend Password As String
End Structure

''' <summary>
''' Ergebnisstruktur fr den Zustand EditPayment.
''' </summary>
Friend Structure EditPaymentResult
    Friend TicketCount As Integer
    Friend AccountNumber As String
    Friend BankNumber As String
End Structure

''' <summary>
''' Ergebnisstruktur fr den Zustand LogOnCustomer.
''' </summary>
''' <remarks>
''' Wenn der Benutzername leer ist, wird der Controller den Benutzer nicht
''' anmelden. Wenn der Benutzername und das Kennwort nicht leer sind und die
''' DoCreateCustomer Markierung auf wahr gesetzt ist, wird der Controller ein
''' neues Kundenkonto anlegen. Wenn der Benutzername und das Kennwort nicht leer
''' sind und die Markierung auf falsch gesetzt ist, wird der Controller den
''' Kunden anmelden.
''' </remarks>
Friend Structure LogOnCustomerResult
    Friend UserName As String
    Friend Password As String
    Friend DoCreateCustomer As Boolean
    Friend UseSampleMode As Boolean
    Friend WebServiceAddress As Uri
End Structure

''' <summary>
''' Ergebnisstruktur fr den Zustand Overview.
''' </summary>
''' <remarks>
''' Die einzelnen Markierungen werden vom Controller in nicht festgelegter
''' Reihenfolge berprft und wenn eine Markietung auf wahr gesetzt ist, wird
''' der Controller den entsprechenden Zustand anzeigen. Falls alle Markierungen
''' auf falsch gesetzt sind, wird der Zustand Overview erneut angezeit werden.
''' </remarks>
Friend Structure OverviewResult
    Friend DoEditCustomer As Boolean
    Friend DoLogOnCustomer As Boolean
    Friend DoLogOffCustomer As Boolean
    Friend DoSearchMoviesByGenres As Boolean
    Friend DoShowMovies As Boolean
    Friend DoShowNews As Boolean
    Friend DoShowPerformances As Boolean
    Friend DoShowTickets As Boolean
End Structure

''' <summary>
''' Ergebnisstruktur fr den Zustand ShowPerformances.
''' </summary>
''' <remarks>
''' Wenn das MovieInfo Feld einen Verweis auf einen Film enthlt, wird der
''' Controller ihn anzeigen. Falls nicht und wenn das PerformanceInfo Feld einen
''' Verweis auf eine Vorstellung enthlt, wird der Controller diese anzeigen.
''' Falls beides nicht zutrifft, wird der Controller den Zustand Overview
''' anzeigen.
''' </remarks>
Friend Structure ShowPerformancesResult
    Friend MovieInfo As MovieInfo
    Friend PerformanceInfo As PerformanceInfo
End Structure