Imports AdventureWorks.Cinema.ServiceAccessLayer.WebService

''' <summary>
''' Implementierung einer Sicht fr einen PocketPC.
''' </summary>
''' <remarks>
''' Diese Implementierung der IView Schnittstelle benutzt je nach Aufruf
''' verschiedene Fensterklassen, um die bergebenen Daten abzuzeigen. Die
''' Methoden geben alle Eingabeparameter an die Fensterobjekte weiter und lesen
''' bentigte Ausgabeparameter von ihnen aus. Alle Methoden haben die folgende
''' Form:
''' 1. Erstelle das Fensterobjekt
''' 2. Gib alle Eingabeparameter an das Objekt weiter
''' 3. Zeige das Fenster modal an (d.h. synchroner Aufruf)
''' 4. Lese die Ausgabeparameter aus dem Objekt aus
''' 5. Gib die Ausgabeparameter an den Aufrufer zurck
''' Zur bergabe von Parametern zwischen der Sicht und den Fensterobjekte
''' werden ffentlich zugngliche Properties benutzt. Nur-Eingabeparameter
''' werden an WriteOnly Properties bergeben, Nur-Ausgabeparameter werden von
''' ReadOnly Properties gelesen. Fr alle anderen Parameter gibt es Properties
''' mit Lese- und Schreibzugriff. Die Fensterklassen verabeiten alle Daten
''' in folgender Reihenfolge:
''' 1. Konstruktor: initialisiere alle privaten Felder mit Standardwerten.
''' 2. Schreibzugriff der Properties: kopiere alle bergebenen Daten in die
''' Objekte der Benutzungsschnittstelle (Knpfe, Listen).
''' 3. Zeige das Fenster an und werte die Eingaben des Benutzers aus. 
''' 4. Ereignisbehandlung der Elemente der Benutzungsschnittstelle: Aktiviere
''' oder deaktiviere Elemente, kopiere ihren Inhalt in private Felder.
''' 5. Lesezugruff der Properties: gib die privaten Felder zurck.
''' </remarks>
Friend Class View
    Implements IView

    ' Feld fr die IsSampleMode Property.
    Private isSampleModeField As Boolean

    ' Property fr den Zustand des Demo-Modus.
    Friend WriteOnly Property IsSampleMode() As Boolean _
        Implements IView.IsSampleMode
        Set(ByVal value As Boolean)
            isSampleModeField = value
        End Set
    End Property

    ''' <summary>
    ''' Der Konstruktor der Sicht.
    ''' </summary>
    ''' <remarks>
    ''' Erstellt ein Fenster mit einem Hintergrundbild, so dass der Benutzer
    ''' nicht den Desktop (oder eine andere Anwendung) sieht, wenn der
    ''' Controller den Bildschirm wechselt (auf langsamen Gerten oder im
    ''' Emulator) oder wenn eine Nachricht angezeigt wird.
    ''' </remarks>
    Friend Sub New()

        ' Erstelle das Fenster mit dem Hintergrundbild.
        Dim form As New FormBackground
        form.Show()

    End Sub

    Friend Sub Message(ByVal text As String) Implements IView.Message

        ' Zeiche ein Nachrichtenfenster an.
        MsgBox(text)

    End Sub

    Friend Function EditPayment(ByVal performance As Performance) _
        As EditPaymentResult Implements IView.EditPayment

        ' Erstelle das Fenster.
        Dim form As New FormEditPayment

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib die Vorstellung.
        form.Performance = performance

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib die Ergebnisstruktur zurck.
        Return form.Result

    End Function

    Function EditCustomer(ByVal customer As User, _
        ByVal genres As List(Of Genre)) As EditCustomerResult _
        Implements IView.EditCustomer

        ' Erstelle das Fenster.
        Dim form As New FormEditCustomer

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib die Genre Liste und den Kunden.
        ' Die Genre Liste mu vor dem Kunden bergeben werden!
        form.Genres = genres
        form.Customer = customer

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib die Ergebnisstruktur zurck.
        Return form.Result

    End Function

    Friend Function LogOnCustomer(ByVal userName As String, _
        ByVal password As String, ByVal webServiceAddress As Uri) _
        As LogOnCustomerResult Implements IView.LogOnCustomer

        ' Erstelle das Fenster.
        Dim form As New FormLogOnCustomer

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib den Benutzernamen und das Kennwort.
        form.UserName = userName
        form.Password = password

        ' bergib die URL des Webservice.
        form.WebServiceAddress = webServiceAddress

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib die Ergebnisstruktur zurck.
        Return form.Result

    End Function

    Friend Function Overview(ByVal customer As User) As OverviewResult _
        Implements IView.Overview

        ' Erstelle das Fenster.
        Dim form As New FormOverview

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib den Kunden.
        form.Customer = customer

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib die Ergebnisstruktur zurck.
        Return form.Result

    End Function

    Friend Function ShowMovie(ByVal movie As Movie) As Boolean _
        Implements IView.ShowMovie

        ' Erstelle das Fenster.
        Dim form As New FormShowMovie

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib den Film.
        form.Movie = movie

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib das Ergebnis zurck.
        Return form.DoShowPerformances

    End Function

    Friend Function ShowMovies(ByVal movies As List(Of MovieInfo)) _
        As MovieInfo Implements IView.ShowMovies

        ' Erstelle das Fenster.
        Dim form As New FormShowMovies

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib die Filmliste.
        form.Movies = movies

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib den ausgewhlten Film zurck.
        Return form.MovieInfo

    End Function

    Friend Function ShowNews(ByVal news As List(Of NewsItem)) As NewsItem _
        Implements IView.ShowNews

        ' Erstelle das Fenster.
        Dim form As New FormShowNews

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib die Nachrichtenliste.
        form.News = news

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib die ausgewhlte Nachricht zurck.
        Return form.NewsItem

    End Function

    Friend Sub ShowNewsItem(ByVal newsItem As NewsItem) _
        Implements IView.ShowNewsItem

        ' Erstelle das Fenster.
        Dim form As New FormShowNewsItem

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib die Nachricht.
        form.NewsItem = newsItem

        ' Zeige das Fenster an.
        form.ShowDialog()

    End Sub

    Friend Function ShowPerformances(ByVal customer As User, _
        ByVal performanceInfos As List(Of PerformanceInfo), _
        ByVal movieInfo As MovieInfo) As ShowPerformancesResult _
        Implements IView.ShowPerformances

        ' Erstelle das Fenster.
        Dim form As New FormShowPerformances

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib den Kunden, die Vorstellung und den Film.
        form.Customer = customer
        form.PerformanceInfos = performanceInfos
        form.MovieInfo = movieInfo

        ' Zeige das Fenster an.
        form.ShowDialog()

        ' Gib die Ergebnisstruktur zurck.
        Return form.Result

    End Function

    Friend Sub ShowTickets(ByVal tickets As List(Of Ticket)) _
        Implements IView.ShowTickets

        ' Erstelle das Fenster.
        Dim form As New FormShowTickets

        ' bergib den Zustand des Demo-Modus.
        form.IsSampleMode = isSampleModeField

        ' bergib die Tickets.
        form.Tickets = tickets

        ' Zeige das Fenster an.
        form.ShowDialog()

    End Sub

End Class