using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace AdventureWorks.Cinema.ServiceAccessLayer.Caching
{
    /// <summary>
    /// Reprsentiert eine Anfrage von
    /// <see cref="IServiceAccessProvider.GetMovies(DateTime)"/>.
    /// </summary>
    internal sealed class MoviesByStartDateCacheRequest : Request
    {
        public MoviesByStartDateCacheRequest(DateTime startDate)
        {
            // Erzeugt ein XML Dokument um eine XML Reprsentation dieser
            // Anfrage zu erzeugen. Das XML Dokument selbst wird 
            // eigentlich nicht bentigt; es wird nur benutzt, um auf eine
            // einfache und sichere Art und Weise einen XML String zu
            // erzeugen.

            XmlDocument doc = new XmlDocument();
            XmlElement root = doc.CreateElement(
                typeof(MoviesByStartDateCacheRequest).Name);
            doc.AppendChild(root);

            XmlElement startDateElement = doc.CreateElement("startDate");
            startDateElement.InnerText = XmlConvert.ToString(startDate, 
                XmlDateTimeSerializationMode.Utc);
            root.AppendChild(startDateElement);

            Xml = doc.InnerXml;
        }
    }
}
