using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace AdventureWorks.Cinema.ServiceAccessLayer.Caching
{
    /// <summary>
    /// Reprsentiert eine Anfrage von
    /// <see cref="IServiceAccessProvider.GetNewsItemsByCategories(int[])"/>.
    /// </summary>
    internal sealed class NewsItemsByCategoriesCacheRequest : Request
    {
        public NewsItemsByCategoriesCacheRequest(int[] numbers)
        {
            // Erzeugt ein XML Dokument um eine XML Reprsentation dieser
            // Anfrage zu erzeugen. Das XML Dokument selbst wird 
            // eigentlich nicht bentigt; es wird nur benutzt, um auf eine
            // einfache und sichere Art und Weise einen XML String zu
            // erzeugen.

            XmlDocument doc = new XmlDocument();
            XmlElement root = doc.CreateElement(
                typeof(NewsItemsByCategoriesCacheRequest).Name);
            doc.AppendChild(root);

            foreach (int number in numbers)
            {
                XmlElement numberElement = doc.CreateElement("number");
                numberElement.InnerText = XmlConvert.ToString(number);
                root.AppendChild(numberElement);
            }

            Xml = doc.InnerText;
        }
    }
}
