using System;
using System.Collections.Generic;
using System.Text;

namespace AdventureWorks.Cinema.ServiceAccessLayer.Caching
{
    /// <summary>
    /// Reprsentiert eine parametrisierten Anfrage. Anfragen rendern sich
    /// selbst als XML String. Das erlaubt einfacheres Debugging, Logging 
    /// und am wichtigsten: Hashing. Zwei Anfragen gelten dann als gleich,
    /// wenn sie beiden den gleichen XML Text haben.
    /// </summary>
    internal abstract class Request
    {
        private string xml;

        /// <summary>
        /// Ruft die die XML Reprsentation dieses Eintrags ab oder setzt
        /// sie. Abgleitete Klassen setzen diese Eigenschaft selbst im
        /// Konstruktor.
        /// </summary>
        public string Xml
        {
            get 
            { 
                return xml;
            }

            protected set
            {
                if (value == null)
                    throw new ArgumentNullException("value");

                xml = value;
            }
        }

        /// <summary>
        /// berschrieben um den HashCode des XML Strings zu liefern.
        /// </summary>
        public override int GetHashCode()
        {
            return Xml.GetHashCode();
        }

        /// <summary>
        /// berschrieben um Wertegleichheit auschlielich auf Basis 
        /// des Wertes von <see cref="Xml"/> durchzufhren.
        /// </summary>
        /// <param name="obj">Das Objekt mit dem verglichen wird..
        /// </param>
        public override bool Equals(object obj)
        {
            if (obj == null)
                return false;

            Request right = obj as Request;

            if (right == null)
                return false;

            return Xml == right.Xml;
        }
    }
}
