using System;
using System.Collections.Generic;
using System.Text;
#if !PocketPC
using System.Runtime.Serialization;
#endif

namespace AdventureWorks.Cinema.ServiceAccessLayer
{
    /// <summary>
    /// <see cref="ServiceAccessException"/> ist die Basiklasse fr alle 
    /// Ausnahmen, die vom <see cref="ServiceAccess"/> geworfen werden und
    /// spezifisch fr den Service Access Layer sind.
    /// </summary>
#if !PocketPC
    [Serializable]
#endif
    public class ServiceAccessException : Exception
#if !PocketPC
        , ISerializable
#endif
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der 
        /// <see cref="ServiceAccessException"/>-Klasse.
        /// </summary>
        public ServiceAccessException()
            : base()
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der 
        /// <see cref="ServiceAccessException"/>-Klasse mit einer angegebenen 
        /// Fehlermeldung.
        /// </summary>
        /// <param name="message">Die Meldung, in der der Fehler beschrieben
        /// wird.</param>
        public ServiceAccessException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der 
        /// <see cref="ServiceAccessException"/>-Klasse mit einer angegebenen 
        /// Fehlermeldung und einem Verweis auf die innere Ausnahme, die diese 
        /// Ausnahme ausgelst hat.
        /// </summary>
        /// <param name="message">Die Fehlermeldung, in der die Ursache der
        /// Ausnahme erklrt wird.</param>
        /// <param name="innerException">Die Ausnahme, die Ursache der aktuellen
        /// Ausnahme ist. Wenn der <paramref name="innerException"/>-Parameter
        /// kein <see langword="null"/>-Verweis ist, wird die aktuelle Ausnahme
        /// in einem <see langword="catch"/>-Block ausgelst, der die
        /// innere Ausnahme behandelt.</param>
        public ServiceAccessException(string message, Exception innerException)
            : base(message, innerException)
        {
        }

#if !PocketPC
        /// <summary>
        /// Initialisiert eine neue Instanz der 
        /// <see cref="ServiceAccessException"/>-Klasse mit serialisierten 
        /// Daten.
        /// </summary>
        /// <param name="info">Das Objekt, das die serialisierten Objektdaten
        /// fr die ausgelste Ausnahme enthlt.</param>
        /// <param name="context">Die Kontextinformationen ber die Quelle oder
        /// das Ziel.</param>
        protected ServiceAccessException(SerializationInfo info, 
            StreamingContext context)
            : base(info, context)
        {
        }
#endif
    }
}
