using System;
using System.Collections.Generic;
using System.Text;

namespace AdventureWorks.Cinema.ServiceAccessLayer
{
    /// <summary>
    /// Diese Klasse dient als Wrapper fr die Designer generierte Klasse
    /// <see 
    /// cref="AdventureWorks.Cinema.ServiceAccessLayer.Properties.Settings"/>,
    /// um den SAL auch auf dem Compact Framework (CF) kompilierbar zu machen.
    /// Leider untersttzt das CF keine .NET Konfigurationsdateien (App.Config).
    /// </summary>
    /// <remarks>
    /// Wenn der SAL auf dem CF kompiliert wird, liefert diese Klasse lediglich
    /// Konstanten. Das ist kein Problem, da diese Einstellungen lediglich
    /// zum Initialisieren von <see cref="ServiceAccess"/> mit Standardwerten
    /// verwendet werden. Alle Einstellungen knnen aber auch ber ffentliche 
    /// Eigenschaften zur Laufzeit verndert werden.
    /// </remarks>
    internal static class Settings
    {
#if PocketPC
        private const string WEB_SERVCE_URL = "http://localhost:1524/" + 
            "CinemaWebService/Service.asmx";
        private const string SAMPLE_DATA_FILENAME = "SampleData.xml";
#endif
        public static string WebServiceUrl
        {
            get
            {
#if PocketPC
                // Auf dem Compact Framework wird einfach eine Konstante 
                // zurckgegeben.
                return WEB_SERVCE_URL;
#else
                // Andernfalls wird die Einstellung aus der Konfigurationsdatei
                // zurckgegeben.
                return AdventureWorks.Cinema.ServiceAccessLayer.Properties
                    .Settings.Default
                    .CinemaAccessService_WebService_CinemaServiceImplementation;
#endif
            }
        }

        public static string SampleDataFileName
        {
            get
            {
#if PocketPC
                // Auf dem Compact Framework wird einfach ein Dateiname
                // relativ zur ausfhrbaren Datei der Anwendung zurckgegeben.
                string executablePath = System.Reflection.Assembly.
                    GetExecutingAssembly().GetName().CodeBase;
                string executableDirectory = System.IO.Path.
                    GetDirectoryName(executablePath);
                return System.IO.Path.Combine(executableDirectory, 
                SAMPLE_DATA_FILENAME);

#else
                // Andernfalls wird die Einstellung aus der Konfigurationsdatei
                // zurckgegeben.
                return AdventureWorks.Cinema.ServiceAccessLayer.Properties
                    .Settings.Default.SampleDataFileName;
#endif
            }
        }

        public static bool IsSampleMode
        {
            get
            {
#if PocketPC
                // Auf dem Compact Framework wird einfach eine Konstante 
                // zurckgegeben.
               return true;
#else
                // Andernfalls wird die Einstellung aus der Konfigurationsdatei
                // zurckgegeben.
                return AdventureWorks.Cinema.ServiceAccessLayer.Properties
                    .Settings.Default.IsSampleMode;
#endif
            }
        }

        public static bool IsCachingEnabled
        {
            get
            {
#if PocketPC
                // Auf dem Compact Framework wird einfach eine Konstante 
                // zurckgegeben.
               return true;
#else
                // Andernfalls wird die Einstellung aus der Konfigurationsdatei
                // zurckgegeben.
                return AdventureWorks.Cinema.ServiceAccessLayer.Properties
                    .Settings.Default.CachingEnabled;
#endif
            }
        }
    }
}
